import { Router, IRequest } from 'itty-router'

type AuthRequest = {
  headers?: Headers
  host?: string
  auth?: string
} & IRequest

interface Issue {
  key: string
  fields: {
    summary: string
    assignee: { displayName: string }
    status: { name: string }
    created: string
  }
}

const router = Router({ base: '/jira-data-source/:host' })

const parseAuth = (request: IRequest) => {
  const { params } = request
  request.host = (params || {}).host
  if (request.headers.has('authorization')) {
    request.auth = request.headers.get('authorization') || ''
  } else {
    return new Response('Unauthorized', { status: 401 })
  }
}

router.get('/', parseAuth, ({ host, auth }: AuthRequest) => {
  return fetch(`https://${host}/rest/api/3/myself`, {
    headers: {
      Authorization: auth ? auth : "",
    },
  })
})

router.all('/search', parseAuth, async ({ host, auth }: AuthRequest) => {
  const response = await fetch(`https://${host}/rest/api/3/filter/favourite`, {
    headers: {
      Authorization: auth ? auth : "",
    },
  })
  const filters = await response.json() as []
  const result = filters.map((filter: { name: string; id: string }) => ({
    text: filter.name,
    value: filter.id,
  }))
  return new Response(JSON.stringify(result), {
    headers: response.headers,
  })
})

router.all('/query', parseAuth, async (request: AuthRequest) => {
  const { host, auth } = request
  if (request.json) {
    const body = await request.json()
    // Convert proper formatted Grafana data into the Jira mess
    const from = new Date(body.range.from)
      .toISOString()
      .replace(/T/, ' ')
      .replace(/:([^:]*)$/, '')
    const to = new Date(body.range.to)
      .toISOString()
      .replace(/T/, ' ')
      .replace(/:([^:]*)$/, '')
    const jql = [`created >= "${from}"`, `created <= "${to}"`]
    const target = body.targets[0]
    if (target.target) {
      jql.push(`filter = "${target.target}"`)
    }

    const url =
      `https://${host}/rest/api/3/search?` +
      new URLSearchParams({
        jql: jql.join(' AND '),
        fields: 'summary,assignee,status,created',
      })

    const response = await fetch(url, {
      headers: {
				Authorization: auth ? auth : "",
      },
    })

    const data = await response.json() as any
    const result = []
    if (target.type == 'timeserie') {
      const datapoints = data.issues.map((issue: Issue) => [
        1,
        new Date(issue.fields.created).getTime(),
      ])
      result.push({
        target: target.target,
        datapoints: datapoints,
      })
    } else if (target.type == 'table') {
      const rows = data.issues.map((issue: Issue) => [
        issue.key,
        issue.fields.summary,
        issue.fields.assignee ? issue.fields.assignee.displayName : '',
        issue.fields.status ? issue.fields.status.name : '',
        issue.fields.created,
      ])
      result.push({
        columns: [
          { text: 'Key', type: 'string' },
          { text: 'Summary', type: 'string' },
          { text: 'Assignee', type: 'string' },
          { text: 'Status', type: 'string' },
          { text: 'Created', type: 'time' },
        ],
        type: 'table',
        rows: rows,
      })
    }

    return new Response(JSON.stringify(result), {
      headers: response.headers,
    })
  } else {
    return new Response('Bad Request', { status: 400 })
  }
})

router.all('*', () => {
  return new Response('Not Found.', { status: 404 })
})

export default router;
